
# q-expansion of newform 1722.2.a.v, downloaded from the LMFDB on 16 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1722
weight = 2
poly_data = [-4, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1, 0], [1, 0], [-1, -1], [-1, 0], [2, -2], [1, 1], [2, 0], [2, 2], [5, -1], [-5, 3], [8, 0], [-3, 5], [1, 0], [-2, -2], [7, -3], [7, -1], [-8, 0], [6, -4], [-11, -1], [-2, 2], [4, -2], [5, -1], [-2, 2], [0, 2], [13, 1], [-6, -4], [-7, 3], [-3, 3], [-6, 0], [-4, -2], [-4, -4], [2, 6], [-5, 3], [-5, 9], [-10, -4], [8, 0], [-2, 8], [-8, 4], [5, 7], [3, 3], [-10, 2], [10, -4], [-10, 2], [2, -8], [-6, 4], [-6, -2], [-17, -3], [3, 1], [-19, 3], [27, -1], [-6, 8], [24, 0], [-10, 4], [14, -6], [0, -6], [2, -10], [11, 3], [7, 5], [17, -7], [-13, 3], [-20, 4], [-14, -4], [9, -13], [-7, 11], [-1, -1], [11, 3], [-15, 11], [-1, 7], [-2, 2], [4, 6], [2, -8], [-9, 13], [-7, -5], [5, -3], [14, -10], [21, -1], [-14, 4], [18, -4], [12, -2], [16, -6], [10, -18], [14, 4], [0, -8], [10, 0], [-28, -4], [-32, 4], [-16, 10], [-4, -2], [-29, 3], [13, -9], [6, -14], [-40, 0], [8, -16], [-8, -4], [-8, 0], [-17, 5], [-24, -2], [-18, -4], [4, 4], [-5, 7], [29, -9], [2, -16], [-9, -15], [4, -18], [-20, -4], [14, -12], [-4, 0], [20, 6], [3, 17], [23, 7], [-32, 0], [-17, 19], [-4, 6], [-19, -9], [28, 4], [2, 0], [8, 4], [2, 6], [14, -4], [-6, -10], [4, 6], [18, 8], [-10, -4], [0, -8], [4, -16], [32, -10], [2, -8], [28, -12], [20, -12], [12, -2], [-10, -2], [-8, 16], [-1, -3], [18, -8], [5, 17], [0, 2], [-16, 22], [1, 11], [14, -20], [19, 11], [-9, -11], [2, 12], [24, -8], [8, 0], [-14, 8], [47, -3], [6, 12], [-23, 13], [13, -1], [19, -7], [10, -20], [3, -5], [15, -11], [29, 7], [-22, 18], [0, -8], [5, -17], [28, -18], [22, 4], [-35, -7], [3, 13], [14, 12], [-9, 5], [23, -23], [6, 12], [20, -12], [-1, 21], [-7, -7], [9, -27], [40, 6], [-8, 24], [-16, -14], [-26, 10], [25, -3], [-32, 16], [16, -14], [-37, 1], [32, -2], [6, 2], [-23, -15], [-64, 0], [-10, -6], [-14, 0], [29, 1], [-43, -1], [26, 0], [-46, -4], [-45, 1], [32, 10], [8, -8], [22, -4], [24, -8], [34, 10], [-14, 8], [23, -7], [-6, 8], [14, 4], [50, -8], [-4, -2], [28, 4], [38, -16], [35, -7], [-46, 4], [15, 7], [-4, 16], [-2, -16], [-17, -3], [20, 12], [-10, 20], [-28, -4], [-18, -4], [12, -14], [-18, 2], [-34, 2], [53, -9], [-52, -2], [5, 15], [-2, 4], [2, -18], [20, -22], [-2, 12], [-53, 9], [1, 13], [33, -13], [40, -16], [-22, 16], [-5, 11], [-15, -13], [-48, 0], [-3, -5], [11, 7], [6, 6], [36, 4], [-46, 8], [20, -16], [0, -16], [5, 9], [29, 1], [-47, -9], [-32, 24], [-38, 6], [14, -18], [28, -12], [-25, 3], [7, 7], [-36, 12], [-19, 23], [1, 7], [-58, -2], [-7, -13], [-31, 17], [-9, 23], [-1, -3], [-10, -12], [2, 12], [15, -15], [8, -14], [30, -2], [-4, -6], [37, -15], [-42, 2], [-14, -18], [-21, -1], [25, 17], [5, 25], [-33, 5], [-66, -4], [-51, 17], [32, -12], [11, 3], [-21, -17], [8, -4], [-4, -26], [-52, -4], [-6, -24], [8, -16], [2, 14], [24, -22], [-52, 6], [43, 13], [40, 0], [19, 17], [-24, 16], [18, 24], [-6, -14], [-22, -18], [38, -12], [-20, 18], [12, -20], [-20, -18], [-29, 19], [-64, 8], [-14, 0], [-34, -6], [10, 16], [34, 12], [50, -2], [-15, -11], [-45, 21], [-62, 10], [-28, -10], [47, -9], [-12, -12], [-26, 2], [-72, -4], [2, -16], [33, 15], [-2, 28], [14, 2], [40, -6], [31, 13], [-9, -1], [36, -10], [16, 12], [63, -11], [-12, -18], [32, -8], [37, -25], [57, 5], [-22, 16], [-15, 27], [31, -1], [66, 8], [-32, -8], [-38, 32], [11, -21], [-25, 29], [0, 36], [-42, -6], [-54, -4], [-24, -22], [24, -18], [-11, 7], [-15, 15], [14, -34], [-9, 33], [-2, 4], [31, 15], [37, 1], [-32, -16], [32, 2], [-28, -2], [40, -10], [8, 8], [16, 14], [-25, 9], [80, -6], [62, -10], [-1, 29], [-18, 28], [38, -26], [67, -5], [5, 9], [-40, -8], [-26, 4], [2, 8], [12, -36], [48, -24], [-28, 22], [-21, -7], [37, -35], [-28, -18], [42, 6], [-1, -31], [-16, 12], [56, -6], [-10, 16], [34, 6], [13, -23], [-38, -10], [-4, -16], [16, 8], [-38, 0], [-9, -3], [42, 0], [-6, 6], [-63, 3], [66, 0], [10, 8], [67, 11], [34, -12], [14, -20], [22, -14], [18, 16], [-58, -2], [42, -10], [-1, -11], [-51, -3], [58, -22], [21, -9], [70, -20], [6, -24], [-38, -18], [-7, -5], [-45, 17], [4, 22], [-34, 10], [-5, 19], [0, 4], [-6, -32], [-4, 6], [-29, 19], [4, 4], [28, -10], [19, -45], [-8, 24], [66, -4], [-34, -12], [16, -4], [-26, -6], [-2, -28], [20, 18], [24, -8], [73, 3], [-50, -20], [-83, 1], [-29, 9], [38, 10], [46, -12], [12, -4], [44, -6], [-38, 36], [77, -1], [21, -37], [16, -6], [-45, -5], [-12, 4], [9, 5], [62, 6], [-65, -3]]
